@@ -1,5 +1,5 @@
 /* Written by Simon Josefsson <simon@yubico.com>.
- * Copyright (c) 2006-2013 Yubico AB
+ * Copyright (c) 2006-2014 Yubico AB
  * Copyright (c) 2011 Tollef Fog Heen <tfheen@err.no>
  * All rights reserved.
  *
@@ -107,6 +107,7 @@ struct cfg
   char *auth_file;
   char *capath;
   char *url;
+  char *urllist;
   char *ldapserver;
   char *ldap_uri;
   char *ldapdn;
@@ -734,6 +735,8 @@ parse_cfg (int flags, int argc, const char **argv, struct cfg *cfg)
 	cfg->capath = (char *) argv[i] + 7;
       if (strncmp (argv[i], "url=", 4) == 0)
 	cfg->url = (char *) argv[i] + 4;
+      if (strncmp (argv[i], "urllist=", 8) == 0)
+	cfg->urllist = (char *) argv[i] + 8;
       if (strncmp (argv[i], "ldapserver=", 11) == 0)
 	cfg->ldapserver = (char *) argv[i] + 11;
       if (strncmp (argv[i], "ldap_uri=", 9) == 0)
@@ -777,6 +780,7 @@ parse_cfg (int flags, int argc, const char **argv, struct cfg *cfg)
       D (("yubi_attr=%s", cfg->yubi_attr ? cfg->yubi_attr : "(null)"));
       D (("yubi_attr_prefix=%s", cfg->yubi_attr_prefix ? cfg->yubi_attr_prefix : "(null)"));
       D (("url=%s", cfg->url ? cfg->url : "(null)"));
+      D (("urllist=%s", cfg->urllist ? cfg->urllist : "(null)"));
       D (("capath=%s", cfg->capath ? cfg->capath : "(null)"));
       D (("token_id_length=%d", cfg->token_id_length));
       D (("mode=%s", cfg->mode == CLIENT ? "client" : "chresp" ));
@@ -804,6 +808,8 @@ pam_sm_authenticate (pam_handle_t * pamh,
   ykclient_t *ykc = NULL;
   struct cfg cfg_st;
   struct cfg *cfg = &cfg_st; /* for DBG macro */
+  size_t templates = 0;
+  char *urls[10];
 
   parse_cfg (flags, argc, argv, cfg);
 
@@ -875,7 +881,44 @@ pam_sm_authenticate (pam_handle_t * pamh,
     ykclient_set_ca_path (ykc, cfg->capath);
 
   if (cfg->url)
-    ykclient_set_url_template (ykc, cfg->url);
+    {
+      rc = ykclient_set_url_template (ykc, cfg->url);
+      if (rc != YKCLIENT_OK)
+	{
+	  DBG (("ykclient_set_url_template() failed (%d): %s",
+		rc, ykclient_strerror (rc)));
+	  retval = PAM_AUTHINFO_UNAVAIL;
+	  goto done;
+	}
+    }
+
+  if (cfg->urllist)
+    {
+      char *saveptr = NULL;
+      char *part = NULL;
+      char *tmpurl = cfg->urllist;
+
+      while ((part = strtok_r(tmpurl, ";", &saveptr)))
+	{
+	  tmpurl = NULL;
+	  if(templates == 10)
+	    {
+	      DBG (("maximum 10 urls supported in list."));
+	      retval = PAM_AUTHINFO_UNAVAIL;
+	      goto done;
+	    }
+	  urls[templates] = strdup(part);
+	  templates++;
+	}
+      rc = ykclient_set_url_bases (ykc, templates, (const char **)urls);
+      if (rc != YKCLIENT_OK)
+	{
+	  DBG (("ykclient_set_url_bases() failed (%d): %s",
+		rc, ykclient_strerror (rc)));
+	  retval = PAM_AUTHINFO_UNAVAIL;
+	  goto done;
+	}
+    }
 
   if (password == NULL)
     {
@@ -1028,6 +1071,14 @@ pam_sm_authenticate (pam_handle_t * pamh,
     }
 
 done:
+  if (templates > 0)
+    {
+      size_t i;
+      for(i = 0; i < templates; i++)
+        {
+	  free(urls[i]);
+        }
+    }
   if (ykc)
     ykclient_done (&ykc);
   if (cfg->alwaysok && retval != PAM_SUCCESS)
